﻿namespace Ozeki
{
    internal class OzHfIOPerformanceEstimator
    {
        private long _currentBytes = 0;
        private long _lastUpdateBytes = 0;
        private DateTime _startTime;
        private DateTime _etaStartTime;
        private Queue<double> _speedHistory = new Queue<double>();
        private Queue<long> _etaHistory = new Queue<long>();
        private long _averageSpeed = 0;
        private TimeSpan _eta = TimeSpan.Zero;
        private long _totalSize;

        public OzHfIOPerformanceEstimator(long size, long start = 0)
        {
            _totalSize = size;
            _currentBytes = start;
            var date = DateTime.UtcNow.ToUniversalTime();
            _startTime = DateTime.UtcNow;
            _etaStartTime = DateTime.UtcNow;
        }

        public OzHfIOPerformanceEstimation CalculateEstimation(long bytesRead)
        {
            var now = DateTime.UtcNow;
            _currentBytes += bytesRead;
            var timeElapsed = now - _startTime;
            var etaTimeElapsed = now - _etaStartTime;


            var progressInPercentage = ((decimal)_currentBytes / _totalSize) * 100;

            if (timeElapsed.TotalMilliseconds > 100)
            {
                var speedInBytesPerMilliSecond = (_currentBytes - _lastUpdateBytes) / timeElapsed.TotalMilliseconds;
                _lastUpdateBytes = _currentBytes;
                if (speedInBytesPerMilliSecond < 0)
                {
                    speedInBytesPerMilliSecond = 0;
                }
                _speedHistory.Enqueue(speedInBytesPerMilliSecond);
                if (_speedHistory.Count > 5)
                {
                    _speedHistory.Dequeue();
                }
                _averageSpeed = Convert.ToInt64(_speedHistory.Average());
                if (_averageSpeed < 0)
                {
                    _averageSpeed = 0;
                }
                _startTime = DateTime.UtcNow;
            }
            if (etaTimeElapsed.TotalMilliseconds > 1000)
            {
                if (bytesRead > 0 && bytesRead < _totalSize)
                {
                    var downloadRate = _averageSpeed * 1000;
                    var remainingBytes = _totalSize - _currentBytes;
                    var timeLeftInSeconds = remainingBytes / downloadRate;

                    if (timeLeftInSeconds > 0)
                    {
                        _etaHistory.Enqueue(timeLeftInSeconds);
                        if (_etaHistory.Count > 5)
                        {
                            _etaHistory.Dequeue();
                        }
                        var averageTimeLeftInSeconds = Convert.ToInt64(_etaHistory.Average());
                        _eta = TimeSpan.FromSeconds(averageTimeLeftInSeconds);
                    }
                }
                _etaStartTime = DateTime.UtcNow;
            }

            return new OzHfIOPerformanceEstimation((int)Math.Floor(progressInPercentage), _averageSpeed *1000, _eta);
        }



    }
}
