# HFDownloader
HFDownloader is a simple and compact, but powerful tool for downloading files from HuggingFace repositories conveniently. It is a single executable file, it does not require any installation procedure. It has no dependencies.

The hfdownloader.exe executable can be run from the command line, and the files to be downloaded can be passed as a command line parameter. The specified files are processed by HFDownloader and downloaded from HuggingFace.co. The downloaded files are then verified to ensure they are intact.
## Project website
This project is sponsored by Ozeki. The project's website can be found at the following link:
https://ozeki-ai-server.com/p_8742-a-tool-to-download-ai-models-from-huggingface.html

## Table of Contents
- [Introduction](#introduction)
- [Features](#features)
- [Installation](#installation)
- [Usage](#usage)
- [Examples](#examples)
- [Parameters](#parameters)
- [Contact](#contact)
- [License](#license)

## Introduction
**Ozeki AI Server** is a powerful software solution for running AI models locally or accessing remote AI services on Windows or Linux. It allows multiple AI models to operate simultaneously on shared hardware, maximizing efficiency and performance. A key feature is its ability to connect AI models to communication channels like **email**, **phone**, and **SMS**, enabling automated responses and streamlined interactions. Unlike subscribing to online AI services, owning your AI system **reduces long-term costs**. Combined with its **privacy**, **scalability**, **reliability**, and **user-friendly interface** Ozeki AI Server is the ultimate tool for leveraging AI in real-world applications while staying budget-friendly.

**More information:** https://ozeki-ai-server.com

## Features
- **Download HuggingFace Repositories**: Download HuggingFace repositories to your local machine. Select a whole repository, just some files or folders. 
- **Verify Files**: HFDownloader automatically verifies the downloaded files if a checksum can be found on HuggingFace.
- **Reliable**: HFDownloader ensures stable downloads, and works great in low-bandwidth and unstable network environments.
- **Multi-Threaded**: Multiple download and verification processes can run simultaneously, making the whole task faster.
- **Detailed Interface**: Follow the progress and events of HFDownloader on the console.
- **Authentication**: Use your HuggingFace Access Token with HFDownloader to download files from gated repositories.
- **Interactive**: Confirm, or change the file selection on the console before starting the download process.
- **Force Mode**: Force HFDownloader to redownload the specified files.
- **Fast Deployment**: Ready to use, self-contained executable file is available to download.
- **Open Source**: This tool is fully open sourced, enabling users to build, inspect, and modify the source code according to their needs.

## Installation

### Option 1: Use the Precompiled Executable (Recommended)

1. Download the precompiled executable from the [Ozeki Website](https://ozeki-ai-server.com/p_8742-a-tool-to-download-ai-models-from-huggingface.html) or the [Releases](https://github.com/ozekiweb/HFDownloader/releases) page on GitHub.
2. Extract the downloaded file, if you downloaded a compressed executable.
3. Run the executable directly:

```bash
./HFDownloader.exe
```

### Option 2: Build the App Using the .NET Tool
1. Clone the repository:

```bash
git clone https://github.com/ozekiweb/HFDownloader
```

2. Navigate to the project directory:

```bash
cd HFDownload
```

3. Build the project using the .NET CLI:

```bash
dotnet build
```

## Usage
To use the app, provide a prompt and optional configuration parameters:
```bash
HFDownload [url] <options>
```
## Examples
### Download full repository:
```bash
hfdownload.exe https://huggingface.co/nvidia/Llama-3.1-Nemotron-70B-Instruct-HF/
```
### Download single file from repository to custom location:
```bash
hfdownload.exe /nvidia/Llama-3.1-Nemotron-70B-Instruct-HF/tokenizer.json -o C:/AIModels
```

### Force download full repository with custom configuration:
```bash
hfdownload.exe /nvidia/Llama-3.1-Nemotron-70B-Instruct-HF/tokenizer.json -f -n 5 -o C:/AIModels -s 500Mb -r 20
```
## Parameters
| Parameter   | Description                                                                | Default Value  |
|-------------|----------------------------------------------------------------------------|----------------|
| `<url>`     | Specifies the URL of the HuggingFace repository or file to be downloaded   | (none)         |
| `-f`        | Specifies if the download should be forced                                 | `False`        |
| `-t <t>`    | Specifies the HuggingFace Access Token                                     | (none)         |
| `-n <n>`    | Specifies how many threads can be used by the downloading process          | `5`            |
| `-o <o>`    | Specifies where the files should be downloaded                             | `C:\AIModels`  |
| `-s <s>`    | Specifies the maximum download speed                                       | Unlimited      |
| `-r <r>`    | specifies how many times should the download retry if it fails        | `20`           |
| `-?`        | Show help and usage information                                            | (none)         |
| `-v`        | Show version information                                                   | (none)         |

## HFDownloader DLL
The core functionalities of HFDownloader are based on the HFDownloader DLL. Use this DLL if you want to integrate the capabilities of HFDownloader into your own software solution. Use the HFDownloadDLL project to build the DLL.

## Contact
For support or questions about the project, please contact:

- **GitHub Issues:** [Open an issue](https://github.com/ozekiweb/HFDownloader/issues)
- **Ozeki:** [Contact us](https://ozeki-ai-server.com/p_4597-contact-us-to-get-ozeki-chat-server.html)

## License
This project is licensed under the [MIT License](LICENSE).
